#!/bin/bash -e


if [[ $OS_VERSION == "CentOS_9" ]]; then
    echo "Dependencies for CentOS 9."
fi

echo "Recieved OS_VERSION is : $OS_VERSION"


# Function to install basic packages
install_basic_packages() {
    echo "================= Installing basic packages ================"
    echo "Recieved Input as $IMAGE_TYPE"

    if [[ $IMAGE_TYPE == "RUNTIME" ]]; then
        if [[ $OS_VERSION == "CentOS_9" ]]; then
            echo "Dependencies install_basic_packages for CentOS 9."
            sed -i '/^\[.*\]/a sslverify=false' /etc/yum.repos.d/centos.repo
            sed -i '/^\[.*\]/a sslverify=false' /etc/yum.repos.d/centos-addons.repo
        fi
        echo "================= Adding some global settings ==================="
        mkdir -p "$HOME/.ssh/"
        echo -e "Host *\n    StrictHostKeyChecking no" > "$HOME/.ssh/config"
        if [[ $OS_VERSION != "CentOS_9" ]]; then
            echo 'APT::Get::Assume-Yes "true";' >> /etc/yum.conf
        fi

        touch "$HOME/.ssh/known_hosts"
        mkdir -p /etc/drydock
        yum install -y sudo
    fi


    if [[ $OS_VERSION == "CentOS_9" ]]; then
        echo "OS version is CentOS 9. Installing packages with --nobest and --allowerasing options."
        sudo yum -y install \
            wget \
            unzip \
            curl \
            openssl \
            which \
            --nobest \
            --allowerasing
    else
        echo "OS version is not CentOS 9. Installing packages without --nobest and --allowerasing options."
        sudo yum -y install \
            wget \
            unzip \
            curl \
            openssl \
            which
    fi

    sudo yum install -y yum-utils
    sudo yum install epel-release -y
    if [[ $OS_VERSION == "CentOS_9" ]]; then
        sudo dnf config-manager --set-enabled crb
        sudo yum install -y python39
    else
        sudo yum install -y centos-release-scl
    fi

    echo "================= Installing Python ==================="
    sudo yum install -y python3-pip
    sudo python3 -m pip install --upgrade pip

    python3 --version

    python3_version=$(python3 --version 2>&1 | cut -d' ' -f2)
    echo "\"python3\": \"$python3_version\"," >> "$VERSION_FILE"
}

# Function to install Python packages
install_python_packages() {
    echo "================= Installing Python packages =================="
    sudo pip3 install virtualenv
    sudo pip3 install cryptography
    sudo pip3 install -q pyOpenSSL

    pip3 show virtualenv | grep Version
}

# Function to add JQ
install_jq() {
    echo "================= Adding JQ========================="
    sudo yum install -y -q jq
    jq --version
    jq_version=$(jq --version | cut -d'-' -f2)
    echo "\"jq\": \"$jq_version\"," >> "$VERSION_FILE"
}

# Function to install Git
install_git() {
    echo "================= Installing Git===================="
    sudo yum install -y git
    git_version=$(git --version | cut -d' ' -f3)
    echo "\"git\": \"$git_version\"," >> "$VERSION_FILE"
}

# Function to add Google Cloud SDK
install_gcloud() {
    echo "================= Adding gcloud  ============"
    sudo echo -e "[google-cloud-cli]\nname=Google Cloud CLI\nbaseurl=https://packages.cloud.google.com/yum/repos/cloud-sdk-el9-x86_64\nenabled=1\ngpgcheck=1\nrepo_gpgcheck=0\ngpgkey=https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg" | sudo tee /etc/yum.repos.d/google-cloud-sdk.repo
    sudo yum install -y google-cloud-cli
    gcloud_version=$(gcloud --version | head -n1 | cut -d' ' -f4)
    echo "\"gcloud\": \"$gcloud_version\"," >> "$VERSION_FILE"
}

# Function to add AWS CLI
install_awscli() {
    echo "================ Adding awscli ===================="
    curl "https://awscli.amazonaws.com/awscli-exe-linux-x86_64.zip" -o "awscliv2.zip"
    unzip awscliv2.zip && sudo ./aws/install
    echo "/usr/local/bin/aws --version"
    echo $(/usr/local/bin/aws --version)
    echo $(aws --version)

    awscli_version=$(aws --version 2>&1 | cut -d' ' -f1 | cut -d'/' -f2)
    echo "\"awscli\": \"$awscli_version\"," >> "$VERSION_FILE"
}

install_azure_cli() {
    echo "================ Adding azure CLI ===================="
    sudo rpm --import https://packages.microsoft.com/keys/microsoft.asc 
    sudo echo -e "[azure-cli]\nname=Azure CLI\nbaseurl=https://packages.microsoft.com/yumrepos/azure-cli\nenabled=1\ngpgcheck=1\ngpgkey=https://packages.microsoft.com/keys/microsoft.asc" | sudo tee /etc/yum.repos.d/azure-cli.repo
    sudo yum -y install azure-cli
    azure_cli_version=$(az --version 2>&1 | awk '/azure-cli/ {print $2}' | head -n1)
}

# Function to install Helm 2
install_helm2() {
    HELM_VERSION=2.16.9
    echo "================= helm 2 cli $HELM_VERSION ======================================"
    curl -sSL -o helm.tar.gz https://get.helm.sh/helm-v"$HELM_VERSION"-linux-amd64.tar.gz
    tar xzvf helm.tar.gz
    chmod +x ./linux-amd64/helm
    chmod +x ./linux-amd64/tiller
    sudo mv ./linux-amd64/helm /usr/local/bin/helm2
    sudo mv ./linux-amd64/tiller /usr/local/bin/tiller
    rm -fr ./linux-amd64
    rm -f helm.tar.gz

    echo "\"helm2\": \"$HELM_VERSION\"," >> "$VERSION_FILE"
}

# Function to install Helm 3
install_helm3() {
    HELM_VERSION="v3.13.1"
    ARCHITECTURE=$(uname -m)

    echo "================= helm 3 cli $HELM_VERSION ======================================"

    if [[ $ARCHITECTURE == "x86_64" ]]; then
        curl -sSL -o helm.tar.gz https://get.helm.sh/helm-$HELM_VERSION-linux-amd64.tar.gz
        tar xzvf helm.tar.gz
        chmod +x ./linux-amd64/helm
        sudo mv ./linux-amd64/helm /usr/local/bin/helm3
        rm -fr ./linux-amd64
        rm -f helm.tar.gz
    else
        curl -sSL -o helm.tar.gz https://get.helm.sh/helm-$HELM_VERSION-linux-arm64.tar.gz
        tar xzvf helm.tar.gz
        chmod +x ./linux-arm64/helm
        sudo mv ./linux-arm64/helm /usr/local/bin/helm3
        rm -fr ./linux-arm64
        rm -f helm.tar.gz
    fi
    
    echo "\"helm3\": \"$HELM_VERSION\"," >> "$VERSION_FILE"
}

install_docker() {
    echo "================= Installing Docker ======================================"
    sudo yum install -y yum-utils
    sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo
    sudo yum install -y docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose-plugin
    if [[ $IMAGE_TYPE != "RUNTIME" ]]; then
        sudo systemctl start docker && sudo systemctl enable docker
    fi
}

install_gitlfs(){
     echo "================= Installing GitLFS ======================================"
    curl -s https://packagecloud.io/install/repositories/github/git-lfs/script.rpm.sh > gitlfs-install.sh
    bash gitlfs-install.sh 
    sudo yum install -y git-lfs
}

install_kubectl(){
     echo "================= Installing Kubectl ======================================"
    KUBECTL_VERSION=$1
    sudo echo -e "[kubernetes]\nname=Kubernetes\nbaseurl=https://pkgs.k8s.io/core:/stable:/v$KUBECTL_VERSION/rpm/\nenabled=1\ngpgcheck=1\ngpgkey=https://pkgs.k8s.io/core:/stable:/v$KUBECTL_VERSION/rpm/repodata/repomd.xml.key" | sudo tee /etc/yum.repos.d/kubernetes.repo
    sudo yum install -y kubectl
}

install_terraform(){
     echo "================= Installing Terraform ======================================"
    sudo yum install -y yum-utils
    sudo yum-config-manager --add-repo https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo
    sudo yum -y install terraform
}

install_packer(){
    sudo yum install -y yum-utils
    sudo yum-config-manager --add-repo https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo
    sudo yum -y install packer
}


install_gcc(){
    GCC_VERSION=$1
    echo "======================= Installing gcc ========================"
    sudo yum install -y yum-utils
    sudo yum install -y epel-release

    if [[ $OS_VERSION == "CentOS_9" ]]; then
        echo "Dependencies install_gcc for CentOS 9."
        sudo yum install dnf-plugins-core
        sudo yum config-manager --enable crb
        sudo yum clean all
        sudo yum makecache
        sudo yum install -y gcc
        sudo yum install -y make automake cmake gdb
        sudo yum groupinstall -y "Development Tools"
    else
        sudo yum install -y centos-release-scl
        sudo yum-config-manager --enable rhel-server-rhscl-$GCC_VERSION-rpms
        sudo rpm --import /etc/pki/rpm-gpg/RPM-GPG-KEY-CentOS-SIG-SCLo
        sudo yum install -y devtoolset-$GCC_VERSION
        scl enable devtoolset-$GCC_VERSION bash
    fi

    gcc --version

    export PATH="/opt/rh/devtoolset-$GCC_VERSION/root/usr/bin:$PATH"
    echo 'export PATH="/opt/rh/devtoolset-8/root/usr/bin:$PATH"' >> /etc/drydock/.env

}

install_clang(){
     echo "================= Installing Clang ======================================"
    CLANG_VERSION=$1
    sudo yum install -y yum-utils
    sudo yum install -y epel-release
    echo "==================== Installing clang ==================="
    if [[ $OS_VERSION == "CentOS_9" ]]; then
        echo "Dependencies install_clang for CentOS 9."
        sudo dnf config-manager --set-enabled crb
        yum install -y clang
        sudo yum install -y llvm-toolset
    else
        sudo yum install -y centos-release-scl
        sudo yum install -y llvm-toolset-$CLANG_VERSION
        scl enable llvm-toolset-$CLANG_VERSION bash
    fi
    # clang --version
}

install_dotnet(){
     echo "================= Installing Dotnet ======================================"
    DOTNET_VERSION=$1
    rpm -Uvh https://packages.microsoft.com/config/centos/7/packages-microsoft-prod.rpm
    yum install -y dotnet-sdk-${DOTNET_VERSION}
}

install_go(){
    export GOVER=$1
    if [[ $OS_VERSION == "CentOS_9" ]]; then
        echo "Dependencies install_go for CentOS 9."
        yum install -y \
        autoconf \
        bison \
        cmake \
        scons \
        binutils \
        gcc \
        glibc-devel
        sudo yum install -y util-linux # for bsdmainutils
    else
        yum install -y \
        autoconf \
        bison \
        cmake \
        scons \
        binutils \
        gcc \
        bzr \
        bsdmainutils \
        glibc-devel
    fi


    echo "================= Install gvm ==================="
    curl -s -L https://raw.githubusercontent.com/moovweb/gvm/master/binscripts/gvm-installer | bash

    export CGO_ENABLED=0
    [[ -s "$HOME/.gvm/scripts/gvm" ]] && source $HOME/.gvm/scripts/gvm || echo "Could not source from $HOME/.gvm/scripts/gvm"
    export CGO_ENABLED=0

    # Install gvm
    echo "================= Install Go $GOVER ==================="
    export GOROOT_BOOTSTRAP=$GOROOT
    gvm install go$GOVER --prefer-binary
    source /root/.gvm/scripts/gvm || echo "Could not source from /root/.gvm/scripts/gvm"
    gvm use go$GOVER
    
    go version

    echo "======= Install go linter ========="
    curl -sSfL https://raw.githubusercontent.com/golangci/golangci-lint/master/install.sh | sh -s -- -b $(go env GOPATH)/bin
    golangci-lint --version
}

install_nvm(){
    export NVM_VERSION=$1
    echo "================= Installing NVM $NVM_VERSION ================"
    curl -sS https://raw.githubusercontent.com/creationix/nvm/v"$NVM_VERSION"/install.sh | bash
}

install_nodejs(){
    NODE_VERSION=$1
	echo "=============== Installing Node $NODE_VERSION ============="
    . /root/.nvm/nvm.sh && nvm install $NODE_VERSION
}

install_gradle(){
    GRADLE_VERSION=$1
    echo "Installing gradle version: $GRADLE_VERSION"
    echo "================ Installing gradle ================="
    wget -nv https://services.gradle.org/distributions/gradle-$GRADLE_VERSION-all.zip
    unzip -qq gradle-$GRADLE_VERSION-all.zip -d /usr/local && rm -f gradle-$GRADLE_VERSION-all.zip
    ln -fs /usr/local/gradle-$GRADLE_VERSION/bin/gradle /usr/bin
    echo "export PATH=\$PATH:/usr/local/gradle-$GRADLE_VERSION/bin" >> /etc/drydock/.env
}

install_apache_maven(){
    APACHE_MAVEN_VERSION=$1
    APACHE_MAVEN_MAJOR_VERSION=$(echo $APACHE_MAVEN_VERSION | cut -d '.' -f1)
    echo "Installing apache-maven version: $APACHE_MAVEN_VERSION"
    echo "================ Installing apache-maven-$APACHE_MAVEN_VERSION ================="
    wget -nv https://archive.apache.org/dist/maven/maven-$APACHE_MAVEN_MAJOR_VERSION/$APACHE_MAVEN_VERSION/binaries/apache-maven-$APACHE_MAVEN_VERSION-bin.tar.gz
    tar xzf apache-maven-$APACHE_MAVEN_VERSION-bin.tar.gz -C /usr/local && rm -f apache-maven-$APACHE_MAVEN_VERSION-bin.tar.gz
    ln -fs /usr/local/apache-maven-$APACHE_MAVEN_VERSION/bin/mvn /usr/bin
    echo "export PATH=\$PATH:/usr/local/apache-maven-$APACHE_MAVEN_VERSION/bin" >> /etc/drydock/.env
    echo "export M2_HOME=/usr/local/apache-maven-$APACHE_MAVEN_VERSION/" >> /etc/drydock/.env
}

install_apache_ant(){
    APACHE_ANT_VERSION=$1
    echo "Installing apache-ant version: $APACHE_ANT_VERSION"
    echo "================ Installing apache-ant-$APACHE_ANT_VERSION ================="
    wget -nv https://archive.apache.org/dist/ant/binaries/apache-ant-$APACHE_ANT_VERSION-bin.tar.gz
    tar xzf apache-ant-$APACHE_ANT_VERSION-bin.tar.gz -C /usr/local && rm -f apache-ant-$APACHE_ANT_VERSION-bin.tar.gz
    ln -fs /usr/local/apache-ant-$APACHE_ANT_VERSION/bin/ant /usr/bin
    echo "export ANT_HOME=/usr/local/apache-ant-$APACHE_ANT_VERSION" >> /etc/drydock/.env
    echo "export PATH=\$PATH:/usr/local/apache-ant-$APACHE_ANT_VERSION/bin" >> /etc/drydock/.env
}

install_jdk(){
    JDK_VERSION=$1
    sudo rpm --import https://yum.corretto.aws/corretto.key
    sudo curl -L -o /etc/yum.repos.d/corretto.repo https://yum.corretto.aws/corretto.repo
    sudo yum install -y java-$JDK_VERSION-amazon-corretto-devel
}